; ##############################################################################
; ##############################################################################
; ##############################################################################
; INITIALISIERUNG																
;																				
ONE_WIRE_INITIALISIERUNG:
	; PORT + PIN definieren				
	cbi (OW_DDR), (OW_PIN_NR)	; AUF EINGANG	
	cbi (OW_PORT),(OW_PIN_NR)	; PULL-UP aus	
	; 1-wire-STATUS	sperren !			
	rcall STATUS_SPERREN_SETZEN	;  STATUS: gesperrt		
	; 1-WIRE CRC Check				
	rcall ONE_WIRE_ROM_CRC7
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; CRC Berechnung bernommen und abgendert von: http://s-huehn.de				
; CRC-Berechnung Application Note 27 von Dallas/Maxim							
;																				
; CRC-ROM																		
; INP: SRAM																		
;																				
ONE_WIRE_ROM_CRC7:
	push XL          ; auf dem Stack sichern            
	push XH
	push r16
	push r17
	push r18
	push r19
	push r20
	push r21
ONE_WIRE_ROM_CRC7_RUN:
	; INITIALISIERUNG
 	ldi XL,low (adr_OW_ROM_ID_BEREICH8+0) ;Zeiger 				
    ldi XH,high(adr_OW_ROM_ID_BEREICH8+0) 
    ldi     r19, 7                          ;7 Bytes berechnen			
    ldi     r20, 0x18                       ;Konstante fr Berechnung
    clr     r17                            ;CRC-Startwert = 0 setzen
ONE_WIRE_CRC7_100: 
	ld      r18, x+                         ;Byte aus Puffer holen
    mov     r21, r18                        ;Byte zwischenspeichern
    ldi     r16, 8                          ;8 Verschiebungen
ONE_WIRE_CRC7_200: 
	eor     r18, r17                      	;CRC berechnen
    ror     r18                             ;Ergebnis in Carry schieben
    mov     r18, r17                      	;letzten CRC-Wert holen
    brcc    ONE_WIRE_CRC7_300                          ;war Ergebnis = 0? ja -> weiter
    eor     r18, r20                        ;sonst CRC-Wert updaten
ONE_WIRE_CRC7_300: 
	ror     r18                             ;neuen CRC-Wert positionieren
    mov     r17, r18          	            ;und speichern
    mov     r18, r21                        ;restliche Bits holen
    bst     r18, 0                          ;LSB sichern, nchste Bitposition
    ror     r18                             ;gesichertes Bit als MSB einsetzen
    bld     r18, 7                          ;(Ersatz fr RR-Befehl)
    mov     r21, r18                        ;und Wert zwischenspeichern
    dec     r16                             ;alle Verschiebungen erledigt?
    brne    ONE_WIRE_CRC7_200               ;nein -> Schleife
    dec     r19                             ;alle Bytes bearbeitet?
    brne    ONE_WIRE_CRC7_100               ;nein -> Schleife
;-------------------------------------------------------------------------------
	LDS temp,(adr_OW_ROM_ID_BEREICH8+7)
	; Korrektur
	STS(adr_OW_ROM_ID_BEREICH8+7),R17
	cp temp,r17
	brne ONE_WIRE_CRC7_ERROR
	SEC										; CARRY=1	
	rjmp ONE_WIRE_CRC7_FERTIG
;-------------------------------------------------------------------------------
ONE_WIRE_CRC7_ERROR:
	; ERROR 
	cbi (LED_OW_ROT_PORT),(LED_OW_ROT_PIN_NR)	; ROT	
	rcall wait_100ms
	sbi (LED_OW_ROT_PORT),(LED_OW_ROT_PIN_NR)	; ROT	
	; nochmal prfen !			
	rjmp ONE_WIRE_ROM_CRC7_RUN
;-------------------------------------------------------------------------------
ONE_WIRE_CRC7_FERTIG:
	pop r21      							; wiederherstellen                	
	pop r20
	pop r19
	pop r18
	pop r17
	pop r16
	pop XH
	pop XL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STATUS_RX_BEREIT_SETZEN:
	; SPERREN						
	ldi temp,(OW_STATUS_RX_BEREIT)
	mov STATUS,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STATUS_SPERREN_SETZEN:
	; SPERREN						
	ldi temp,(OW_STATUS_GESPERRT)
	mov STATUS,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STATUS_TX_0SLOT_SETZEN:
	; STATUS		
	ldi temp,(OW_STATUS_TX_0SLOT)
	mov STATUS,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STATUS_TX_1SLOT_SETZEN:
	; STATUS		
	ldi temp,(OW_STATUS_TX_1SLOT)
	mov STATUS,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; CARRY bleibt erhalen															
STATUS_TX_SLOT_SETZEN_C:
	brcc STATUS_TX_SLOT_SETZEN_C_CC
	ldi temp,(OW_STATUS_TX_1SLOT)
	mov STATUS,temp	
	ret
STATUS_TX_SLOT_SETZEN_C_CC:
	ldi temp,(OW_STATUS_TX_0SLOT)
	mov STATUS,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

